package com.navarsete.stand012.dummy;

import org.axonframework.commandhandling.annotation.CommandHandler;
import org.axonframework.eventsourcing.annotation.AbstractAnnotatedAggregateRoot;
import org.axonframework.eventsourcing.annotation.AggregateIdentifier;
import org.axonframework.eventsourcing.annotation.EventSourcingHandler;

/**
 *
 * @author viggo
 */
public class SupplierOrder extends AbstractAnnotatedAggregateRoot {
    
    @AggregateIdentifier
    String supplierOrderId;
    
    // TODO Add properties of the SupplierOrder (use semantic vocabularies???)
    private String description;
    
    // Constructor needed for reconstruction
    public SupplierOrder() {
    }
    
    @CommandHandler
    public SupplierOrder(CreateSupplierOrderCommand command) {        
        apply( new SupplierOrderCreatedEvent(command.getPurchaseOrderId(), command.getDescription()));        
    }
    
    /**
     * L1 - Ordreprosess leverandør
     * "Ordremottak"
     * 
     * @param supplierOrderCreatedEvent 
     */
    @EventSourcingHandler
    public void handlePurchaseOrderCreatedEvent(SupplierOrderCreatedEvent supplierOrderCreatedEvent) {      
        this.supplierOrderId = supplierOrderCreatedEvent.getSupplierOrderId();
        this.description = supplierOrderCreatedEvent.getDescription();
    }    
    
    /**
     * L1 - Ordreprosess leverandør
     * "Send ordrebekreftelse"
     * 
     * @param command
     */
    @CommandHandler
    public void sendOrderConfirmation(SendOrderConfirmationCommand command) {
      apply( new OrderConfirmationSentEvent(command.getSupplierOrderId()));
    }
    
   /**
    * L2 - Vareleveranse
    * "Plukke varer"
    * 
     * @param command
    */
    @CommandHandler
    public void pickItems(PickItemsCommand command) {
        apply( new ItemsPickedEvent(command.getSupplierOrderId()));
    }
   
   /**
    * L2 - Vareleveranse
    * "Registerere antall pallplasser og palltyper"
    * 
     * @param command
    */
    @CommandHandler
    public void registerPalletInformation(RegisterPalletInformationCommand command) {
        apply( new PalletInformationRegisteredEvent(command.getSupplierOrderId()));
    }
   
   /**
    * L2 - Vareleveranse
    * "Send pakkseddel"
    */   
    @CommandHandler
    public void sendAdvancedShippingNotice(SendAdvancedShippingNoticeCommand command) {
        apply( new AdvancedShippingNoticeSentEvent(command.getSupplierOrderId()));
    }    
}
